@@ -25,6 +25,7 @@
 #include <string.h>
 #include <glade/glade.h>
 #include "eggcellrendererkeys.h"
+#include "eggaccelerators.h"
 
 #define D(x)
 
@@ -43,6 +44,10 @@
 #define KEY_NEXT_TAB CONF_KEYS_PREFIX"/next_tab"
 #define KEY_SET_TERMINAL_TITLE CONF_KEYS_PREFIX"/set_window_title"
 #define PREFIX_KEY_SWITCH_TO_TAB CONF_KEYS_PREFIX"/switch_to_tab_"
+#define KEY_HELP CONF_KEYS_PREFIX"/help"
+#define KEY_ZOOM_IN CONF_KEYS_PREFIX"/zoom_in"
+#define KEY_ZOOM_OUT CONF_KEYS_PREFIX"/zoom_out"
+#define KEY_ZOOM_NORMAL CONF_KEYS_PREFIX"/zoom_normal"
 
 typedef struct
 {
@@ -92,6 +97,12 @@ static KeyEntry view_entries[] =
     KEY_TOGGLE_MENUBAR, ACCEL_PATH_TOGGLE_MENUBAR, 0, 0, NULL, FALSE },
   { N_("Full Screen"),
     KEY_FULL_SCREEN, ACCEL_PATH_FULL_SCREEN, 0, 0, NULL, FALSE },
+  { N_("Zoom In"),
+    KEY_ZOOM_IN, ACCEL_PATH_ZOOM_IN, 0, 0, NULL, FALSE },
+  { N_("Zoom Out"),
+    KEY_ZOOM_OUT, ACCEL_PATH_ZOOM_OUT, 0, 0, NULL, FALSE },
+  { N_("Normal Size"),
+    KEY_ZOOM_NORMAL, ACCEL_PATH_ZOOM_NORMAL, 0, 0, NULL, FALSE }
 };
 
 static KeyEntry terminal_entries[] =
@@ -148,13 +159,18 @@ static KeyEntry go_entries[] =
     PREFIX_ACCEL_PATH_SWITCH_TO_TAB"12", 0, 0, NULL, FALSE }
 };
 
+static KeyEntry help_entries[] = {
+  { N_("Contents"), KEY_HELP, ACCEL_PATH_HELP, 0, 0, NULL, FALSE}
+};
+
 static KeyEntryList all_entries[] =
 {
   { file_entries, G_N_ELEMENTS (file_entries), N_("File") },
   { edit_entries, G_N_ELEMENTS (edit_entries), N_("Edit") },
   { view_entries, G_N_ELEMENTS (view_entries), N_("View") },
   { terminal_entries, G_N_ELEMENTS (terminal_entries), N_("Terminal") },
-  { go_entries, G_N_ELEMENTS (go_entries), N_("Go") }
+  { go_entries, G_N_ELEMENTS (go_entries), N_("Go") },
+  { help_entries, G_N_ELEMENTS (help_entries), N_("Help") }
 };
 
 enum
@@ -635,6 +651,8 @@ binding_from_string (const char      *str,
                      guint           *accelerator_key,
                      GdkModifierType *accelerator_mods)
 {
+  EggVirtualModifierType virtual;
+  
   g_return_val_if_fail (accelerator_key != NULL, FALSE);
   
   if (str == NULL || (str && strcmp (str, "disabled") == 0))
@@ -644,8 +662,20 @@ binding_from_string (const char      *str,
       return TRUE;
     }
 
-  gtk_accelerator_parse (str, accelerator_key, accelerator_mods);
+  if (!egg_accelerator_parse_virtual (str, accelerator_key, &virtual))
+    return FALSE;
 
+  egg_keymap_resolve_virtual_modifiers (gdk_keymap_get_default (),
+                                        virtual,
+                                        accelerator_mods);
+
+  /* Be sure the GTK accelerator system will be able to handle this
+   * accelerator. Be sure to allow no-accelerator accels like F1.
+   */
+  if ((*accelerator_mods & gtk_accelerator_get_default_mod_mask ()) == 0 &&
+      *accelerator_mods != 0)
+    return FALSE;
+  
   if (*accelerator_key == 0)
     return FALSE;
   else
@@ -881,6 +911,7 @@ accel_edited_callback (GtkCellRendererText *cell,
                        const char          *path_string,
                        guint                keyval,
                        GdkModifierType      mask,
+                       guint                hardware_keycode,
                        gpointer             data)
 {
   GtkTreeModel *model = (GtkTreeModel *)data;
@@ -981,6 +1012,61 @@ disable_menu_accels_toggled (GtkWidget *button,
     }
 }
 
+typedef struct
+{
+  GtkTreeView *tree_view;
+  GtkTreePath *path;
+} IdleData;
+
+static gboolean
+real_start_editing_cb (IdleData *idle_data)
+{
+  gtk_widget_grab_focus (GTK_WIDGET (idle_data->tree_view));
+  gtk_tree_view_set_cursor (idle_data->tree_view,
+                            idle_data->path,
+			    gtk_tree_view_get_column (idle_data->tree_view, 1),
+			    TRUE);
+
+  gtk_tree_path_free (idle_data->path);
+  g_free (idle_data);
+
+  return FALSE;
+}
+
+gboolean
+start_editing_cb (GtkTreeView    *tree_view,
+                  GdkEventButton *event,
+		  gpointer        data)
+{
+  GtkTreePath *path;
+
+  if (event->window != gtk_tree_view_get_bin_window (tree_view))
+    return FALSE;
+
+  if (gtk_tree_view_get_path_at_pos (tree_view,
+                                     (gint) event->x,
+				     (gint) event->y,
+				     &path, NULL,
+				     NULL, NULL))
+    {
+      IdleData *idle_data;
+
+      if (gtk_tree_path_get_depth (path) == 1)
+        {
+	  gtk_tree_path_free (path);
+	  return FALSE;
+	}
+
+      idle_data = g_new (IdleData, 1);
+      idle_data->tree_view = tree_view;
+      idle_data->path = path;
+      g_signal_stop_emission_by_name (G_OBJECT (tree_view), "button_press_event");
+      g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
+    }
+
+  return TRUE;
+}
+
 GtkWidget*
 terminal_edit_keys_dialog_new (GtkWindow *transient_parent)
 {
@@ -1064,13 +1150,11 @@ terminal_edit_keys_dialog_new (GtkWindow *transient_parent)
 
   living_treeviews = g_slist_prepend (living_treeviews, w);
 
+  g_signal_connect (G_OBJECT (w), "button_press_event",
+		    G_CALLBACK (start_editing_cb), NULL);
   g_signal_connect (G_OBJECT (w), "destroy",
                     G_CALLBACK (remove_from_list_callback),
                     &living_treeviews);
-  
-  gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (w)),
-                               GTK_SELECTION_NONE);
-
 
   tree = gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
   
@@ -1086,6 +1170,7 @@ terminal_edit_keys_dialog_new (GtkWindow *transient_parent)
   /* Column 2 */
   cell_renderer = g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
 				"editable", TRUE,
+				"accel_mode", EGG_CELL_RENDERER_KEYS_MODE_GTK,
 				NULL);
   g_signal_connect (G_OBJECT (cell_renderer), "keys_edited",
                     G_CALLBACK (accel_edited_callback),
